/*
 * ngIRCd -- The Next Generation IRC Daemon
 * Copyright (c)2001,2002 by Alexander Barton (alex@barton.de)
 *
 * Dieses Programm ist freie Software. Sie koennen es unter den Bedingungen
 * der GNU General Public License (GPL), wie von der Free Software Foundation
 * herausgegeben, weitergeben und/oder modifizieren, entweder unter Version 2
 * der Lizenz oder (wenn Sie es wuenschen) jeder spaeteren Version.
 * Naehere Informationen entnehmen Sie bitter der Datei COPYING. Eine Liste
 * der an ngIRCd beteiligten Autoren finden Sie in der Datei AUTHORS.
 *
 * $Id: log.c,v 1.19 2002/03/03 17:17:01 alex Exp $
 *
 * log.c: Logging-Funktionen
 *
 * $Log: log.c,v $
 * Revision 1.19  2002/03/03 17:17:01  alex
 * - strncpy() und vsnprintf() kopieren nun etwas "optimierter" (1 Byte weniger) :-)
 *
 * Revision 1.18  2002/02/19 20:07:13  alex
 * - direkt nach dem Start werden die aktiven "Modes" ins Log geschrieben.
 *
 * Revision 1.17  2002/01/11 14:45:37  alex
 * - Anpassungen an neue Kommandozeilen-Optionen "--debug" und "--nodaemon".
 *
 * Revision 1.16  2002/01/05 15:54:40  alex
 * - syslog() etc. wurde verwendet, auch wenn USE_SYSLOG nicht definiert war.
 *
 * Revision 1.15  2002/01/02 02:42:58  alex
 * - Copyright-Texte aktualisiert.
 *
 * Revision 1.14  2002/01/01 18:01:43  alex
 * - Architektur und Betriebssystem in Start-Meldung aufgenommen.
 *
 * Revision 1.13  2001/12/31 02:18:51  alex
 * - viele neue Befehle (WHOIS, ISON, OPER, DIE, RESTART),
 * - neuen Header "defines.h" mit (fast) allen Konstanten.
 * - Code Cleanups und viele "kleine" Aenderungen & Bugfixes.
 *
 * Revision 1.12  2001/12/29 20:16:31  alex
 * - Log-Funktionen fuer Resolver-Sub-Prozess implementiert.
 *
 * Revision 1.11  2001/12/29 03:08:49  alex
 * - neue configure-Option "--enable-strict-rfc".
 *
 * Revision 1.10  2001/12/27 01:44:49  alex
 * - die Verwendung von syslog kann nun abgeschaltet werden.
 *
 * Revision 1.9  2001/12/26 03:22:16  alex
 * - string.h wird nun includiert.
 *
 * Revision 1.8  2001/12/25 23:13:00  alex
 * - Versionsstring bei Programmstart verbessert.
 *
 * Revision 1.7  2001/12/25 22:04:26  alex
 * - Aenderungen an den Debug- und Logging-Funktionen.
 *
 * Revision 1.6  2001/12/25 19:20:39  alex
 * - es wird nun die Facility LOG_LOCAL5 zum Loggen verwendet.
 *
 * Revision 1.5  2001/12/15 00:07:56  alex
 * - Log-Level der Start- und Stop-Meldungen angehoben.
 *
 * Revision 1.4  2001/12/13 02:04:16  alex
 * - boesen "Speicherschiesser" in Log() gefixt.
 *
 * Revision 1.3  2001/12/12 23:31:24  alex
 * - Zum Loggen wird nun auch syslog verwendet.
 *
 * Revision 1.2  2001/12/12 17:19:12  alex
 * - in Log-Meldungen wird nun auch der Level der Meldung ausgegeben.
 *
 * Revision 1.1.1.1  2001/12/11 21:53:04  alex
 * - Imported sources to CVS.
 */


#define MAX_LOG_MSG_LEN 256


#include <portab.h>
#include "global.h"

#include <imp.h>
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#ifdef USE_SYSLOG
#include <syslog.h>
#endif

#include "global.h"
#include "ngircd.h"

#include <exp.h>
#include "log.h"


GLOBAL VOID Log_Init( VOID )
{
	CHAR txt[127];

#ifdef USE_SYSLOG
	/* Syslog initialisieren */
	openlog( PACKAGE, LOG_CONS|LOG_PID, LOG_LOCAL5 );
#endif

	/* Hello World! */
	Log( LOG_NOTICE, "%s started.", NGIRCd_Version( ));

	/* Informationen uebern den "Operation Mode" */
	strcpy( txt, "" );
#ifdef DEBUG
	if( NGIRCd_Debug )
	{
		if( txt[0] ) strcat( txt, ", " );
		strcat( txt, "debug-mode" );
	}
#endif
	if( NGIRCd_NoDaemon )
	{
		if( txt[0] ) strcat( txt, ", " );
		strcat( txt, "no-daemon-mode" );
	}
	if( NGIRCd_Passive )
	{
		if( txt[0] ) strcat( txt, ", " );
		strcat( txt, "passive-mode" );
	}
#ifdef SNIFFER
	if( NGIRCd_Sniffer )
	{
		if( txt[0] ) strcat( txt, ", " );
		strcat( txt, "network sniffer" );
	}
#endif
	if( txt[0] ) Log( LOG_INFO, "Activating: %s.", txt );
} /* Log_Init */


GLOBAL VOID Log_Exit( VOID )
{
	/* Good Bye! */
	Log( LOG_NOTICE, PACKAGE" done.");
	
#ifdef USE_SYSLOG
	/* syslog abmelden */
	closelog( );
#endif
} /* Log_Exit */


GLOBAL VOID Log( CONST INT Level, CONST CHAR *Format, ... )
{
	/* Eintrag in Logfile(s) schreiben */

	CHAR msg[MAX_LOG_MSG_LEN];
	va_list ap;

	assert( Format != NULL );

#ifdef DEBUG
	if(( Level == LOG_DEBUG ) && ( ! NGIRCd_Debug )) return;
#else
	if( Level == LOG_DEBUG ) return;
#endif

	/* String mit variablen Argumenten zusammenbauen ... */
	va_start( ap, Format );
	vsnprintf( msg, MAX_LOG_MSG_LEN, Format, ap );
	va_end( ap );

	/* ... und ausgeben */
	if( NGIRCd_NoDaemon ) printf( "[%d] %s\n", Level, msg );
#ifdef USE_SYSLOG
	syslog( Level, msg );
#endif
} /* Log */


GLOBAL VOID Log_Init_Resolver( VOID )
{
#ifdef USE_SYSLOG
	openlog( PACKAGE, LOG_CONS|LOG_PID, LOG_LOCAL5 );
#endif
} /* Log_Init_Resolver */


GLOBAL VOID Log_Exit_Resolver( VOID )
{
#ifdef USE_SYSLOG
	closelog( );
#endif
} /* Log_Exit_Resolver */


GLOBAL VOID Log_Resolver( CONST INT Level, CONST CHAR *Format, ... )
{
	/* Eintrag des Resolver in Logfile(s) schreiben */

#ifndef USE_SYSLOG
	return;
#else

	CHAR msg[MAX_LOG_MSG_LEN];
	va_list ap;

	assert( Format != NULL );

#ifdef DEBUG
	if(( Level == LOG_DEBUG ) && ( ! NGIRCd_Debug )) return;
#else
	if( Level == LOG_DEBUG ) return;
#endif

	/* String mit variablen Argumenten zusammenbauen ... */
	va_start( ap, Format );
	vsnprintf( msg, MAX_LOG_MSG_LEN, Format, ap );
	va_end( ap );

	/* ... und ausgeben */
	syslog( Level, msg );

#endif
} /* Log_Resolver */


/* -eof- */
